@@ -45,9 +45,9 @@ use cl3::sampler;
 #[allow(unused_imports)]
 use cl3::types::{
     cl_addressing_mode, cl_bool, cl_buffer_create_type, cl_buffer_region, cl_context,
-    cl_filter_mode, cl_image_desc, cl_image_format, cl_int, cl_mem, cl_mem_flags,
-    cl_mem_object_type, cl_mem_properties, cl_sampler, cl_sampler_properties, cl_uint, cl_ulong,
-    CL_FALSE,
+    cl_filter_mode, cl_image_desc, cl_image_format, cl_image_info, cl_int, cl_mem, cl_mem_flags,
+    cl_mem_info, cl_mem_object_type, cl_mem_properties, cl_pipe_info, cl_sampler, cl_sampler_info,
+    cl_sampler_properties, cl_uint, cl_ulong, CL_FALSE,
 };
 use libc::{c_void, intptr_t, size_t};
 use std::mem;
@@ -58,54 +58,57 @@ pub trait ClMem {
     fn get_mut(&mut self) -> cl_mem;
 
     fn mem_type(&self) -> Result<cl_mem_object_type> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_TYPE)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_TYPE)?.into())
     }
 
     fn flags(&self) -> Result<cl_mem_flags> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_FLAGS)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_FLAGS)?.into())
     }
 
     fn size(&self) -> Result<size_t> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_SIZE)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_SIZE)?.into())
     }
 
     fn host_ptr(&self) -> Result<intptr_t> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_HOST_PTR)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_HOST_PTR)?.into())
     }
 
     fn map_count(&self) -> Result<cl_uint> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_MAP_COUNT)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_MAP_COUNT)?.into())
     }
 
     fn reference_count(&self) -> Result<cl_uint> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_REFERENCE_COUNT)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_REFERENCE_COUNT)?.into())
     }
 
     fn context(&self) -> Result<cl_context> {
-        Ok(intptr_t::from(memory::get_mem_object_info(
-            self.get(),
-            MemInfo::CL_MEM_CONTEXT,
-        )?) as cl_context)
+        Ok(intptr_t::from(memory::get_mem_object_info(self.get(), CL_MEM_CONTEXT)?) as cl_context)
     }
 
     fn associated_memobject(&self) -> Result<cl_mem> {
         Ok(intptr_t::from(memory::get_mem_object_info(
             self.get(),
-            MemInfo::CL_MEM_ASSOCIATED_MEMOBJECT,
+            CL_MEM_ASSOCIATED_MEMOBJECT,
         )?) as cl_mem)
     }
 
     fn offset(&self) -> Result<size_t> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_OFFSET)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_OFFSET)?.into())
     }
 
     fn uses_svm_pointer(&self) -> Result<cl_uint> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_USES_SVM_POINTER)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_USES_SVM_POINTER)?.into())
     }
 
     /// CL_VERSION_3_0
     fn properties(&self) -> Result<Vec<cl_ulong>> {
-        Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_PROPERTIES)?.into())
+        Ok(memory::get_mem_object_info(self.get(), CL_MEM_PROPERTIES)?.into())
+    }
+
+    /// Get memory data about an OpenCL memory object.
+    /// Calls clGetMemObjectInfo to get the desired data about the memory object.
+    fn get_mem_data(&self, param_name: cl_mem_info) -> Result<Vec<u8>> {
+        Ok(get_mem_object_data(self.get(), param_name)?)
     }
 
     /// Query an OpenGL object used to create an OpenCL memory object.  
@@ -618,64 +621,72 @@ impl Image {
     }
 
     pub fn format(&self) -> Result<Vec<cl_image_format>> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_FORMAT)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_FORMAT)?.into())
     }
 
     pub fn element_size(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_ELEMENT_SIZE)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_ELEMENT_SIZE)?.into())
     }
 
     pub fn row_pitch(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_ROW_PITCH)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_ROW_PITCH)?.into())
     }
 
     pub fn slice_pitch(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_SLICE_PITCH)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_SLICE_PITCH)?.into())
     }
 
     pub fn width(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_WIDTH)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_WIDTH)?.into())
     }
 
     pub fn height(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_HEIGHT)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_HEIGHT)?.into())
     }
 
     pub fn depth(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_DEPTH)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_DEPTH)?.into())
     }
     pub fn array_size(&self) -> Result<size_t> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_ARRAY_SIZE)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_ARRAY_SIZE)?.into())
     }
 
     pub fn buffer(&self) -> Result<cl_mem> {
-        Ok(intptr_t::from(memory::get_image_info(
-            self.image,
-            ImageInfo::CL_IMAGE_BUFFER,
-        )?) as cl_mem)
+        Ok(intptr_t::from(memory::get_image_info(self.image, CL_IMAGE_BUFFER)?) as cl_mem)
     }
 
     pub fn num_mip_levels(&self) -> Result<cl_uint> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_NUM_MIP_LEVELS)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_NUM_MIP_LEVELS)?.into())
     }
 
     pub fn num_samples(&self) -> Result<cl_uint> {
-        Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_NUM_SAMPLES)?.into())
+        Ok(memory::get_image_info(self.image, CL_IMAGE_NUM_SAMPLES)?.into())
+    }
+
+    /// Get data about an OpenCL image object.
+    /// Calls clGetImageInfo to get the desired data about the image object.
+    pub fn get_data(&self, param_name: cl_image_info) -> Result<Vec<u8>> {
+        Ok(get_image_data(self.image, param_name)?)
     }
 
     ///  Get information about the GL texture target associated with a memory object.
     pub fn gl_texture_target(&self) -> Result<cl_uint> {
-        Ok(gl::get_gl_texture_info(self.image, gl::TextureInfo::CL_GL_TEXTURE_TARGET)?.into())
+        Ok(gl::get_gl_texture_info(self.image, gl::CL_GL_TEXTURE_TARGET)?.into())
     }
 
     /// Get information about the GL mipmap level associated with a memory object.
     pub fn gl_mipmap_level(&self) -> Result<cl_int> {
-        Ok(gl::get_gl_texture_info(self.image, gl::TextureInfo::CL_GL_MIPMAP_LEVEL)?.into())
+        Ok(gl::get_gl_texture_info(self.image, gl::CL_GL_MIPMAP_LEVEL)?.into())
     }
 
     ///  Get information about the GL number of samples associated with a memory object.
     pub fn gl_num_samples(&self) -> Result<cl_int> {
-        Ok(gl::get_gl_texture_info(self.image, gl::TextureInfo::CL_GL_NUM_SAMPLES)?.into())
+        Ok(gl::get_gl_texture_info(self.image, gl::CL_GL_NUM_SAMPLES)?.into())
+    }
+
+    ///  Get GL texture information associated with a memory object.
+    pub fn get_gl_texture_data(&self, param_name: gl::cl_gl_texture_info) -> Result<Vec<u8>> {
+        Ok(gl::get_gl_texture_data(self.image, param_name)?)
     }
 }
 
@@ -736,51 +747,39 @@ impl Sampler {
     }
 
     pub fn reference_count(&self) -> Result<cl_uint> {
-        Ok(
-            sampler::get_sampler_info(
-                self.get(),
-                sampler::SamplerInfo::CL_SAMPLER_REFERENCE_COUNT,
-            )?
-            .into(),
-        )
+        Ok(sampler::get_sampler_info(self.get(), sampler::CL_SAMPLER_REFERENCE_COUNT)?.into())
     }
 
     pub fn context(&self) -> Result<cl_context> {
         Ok(intptr_t::from(sampler::get_sampler_info(
             self.get(),
-            sampler::SamplerInfo::CL_SAMPLER_CONTEXT,
+            sampler::CL_SAMPLER_CONTEXT,
         )?) as cl_context)
     }
 
     pub fn normalized_coords(&self) -> Result<bool> {
         Ok(cl_uint::from(sampler::get_sampler_info(
             self.get(),
-            sampler::SamplerInfo::CL_SAMPLER_NORMALIZED_COORDS,
+            sampler::CL_SAMPLER_NORMALIZED_COORDS,
         )?) != CL_FALSE)
     }
 
     pub fn addressing_mode(&self) -> Result<cl_addressing_mode> {
-        Ok(
-            sampler::get_sampler_info(
-                self.get(),
-                sampler::SamplerInfo::CL_SAMPLER_ADDRESSING_MODE,
-            )?
-            .into(),
-        )
+        Ok(sampler::get_sampler_info(self.get(), sampler::CL_SAMPLER_ADDRESSING_MODE)?.into())
     }
 
     pub fn filter_mode(&self) -> Result<cl_filter_mode> {
-        Ok(
-            sampler::get_sampler_info(self.get(), sampler::SamplerInfo::CL_SAMPLER_FILTER_MODE)?
-                .into(),
-        )
+        Ok(sampler::get_sampler_info(self.get(), sampler::CL_SAMPLER_FILTER_MODE)?.into())
     }
 
     pub fn sampler_properties(&self) -> Result<Vec<intptr_t>> {
-        Ok(
-            sampler::get_sampler_info(self.get(), sampler::SamplerInfo::CL_SAMPLER_PROPERTIES)?
-                .into(),
-        )
+        Ok(sampler::get_sampler_info(self.get(), sampler::CL_SAMPLER_PROPERTIES)?.into())
+    }
+
+    /// Get data about an OpenCL sampler object.
+    /// Calls clGetDeviceInfo to get the desired data about the sampler object.
+    pub fn get_data(&self, param_name: cl_sampler_info) -> Result<Vec<u8>> {
+        Ok(sampler::get_sampler_data(self.get(), param_name)?)
     }
 }
 
@@ -794,6 +793,14 @@ pub struct Pipe {
     pipe: cl_mem,
 }
 
+#[cfg(feature = "CL_VERSION_2_0")]
+impl From<cl_mem> for Pipe {
+    fn from(pipe: cl_mem) -> Self {
+        Pipe { pipe }
+    }
+}
+
+#[cfg(feature = "CL_VERSION_2_0")]
 impl From<Pipe> for cl_mem {
     fn from(value: Pipe) -> Self {
         value.pipe as cl_mem
@@ -835,15 +842,21 @@ impl Pipe {
     }
 
     pub fn pipe_packet_size(&self) -> Result<cl_uint> {
-        Ok(memory::get_pipe_info(self.get(), PipeInfo::CL_PIPE_PACKET_SIZE)?.into())
+        Ok(memory::get_pipe_info(self.get(), CL_PIPE_PACKET_SIZE)?.into())
     }
 
     pub fn pipe_max_packets(&self) -> Result<cl_uint> {
-        Ok(memory::get_pipe_info(self.get(), PipeInfo::CL_PIPE_MAX_PACKETS)?.into())
+        Ok(memory::get_pipe_info(self.get(), CL_PIPE_MAX_PACKETS)?.into())
     }
 
     pub fn pipe_properties(&self) -> Result<Vec<intptr_t>> {
-        Ok(memory::get_pipe_info(self.get(), PipeInfo::CL_PIPE_PROPERTIES)?.into())
+        Ok(memory::get_pipe_info(self.get(), CL_PIPE_PROPERTIES)?.into())
+    }
+
+    /// Get data about an OpenCL pipe object.  
+    /// Calls clGetPipeInfo to get the desired information about the pipe object.
+    pub fn get_data(&self, param_name: cl_pipe_info) -> Result<Vec<u8>> {
+        Ok(memory::get_pipe_data(self.get(), param_name)?)
     }
 }
 
